@@ -3,6 +3,8 @@
 # and distributed under the terms of the GNU General Public Licence,
 # either version 2, or (at your option) any later version.
 
+import fnmatch
+import getpass
 import glob
 import os
 import re
@@ -19,7 +21,7 @@ from . import conf
 from . import connection
 from . import core
 from . import oscerr
-from .core import get_buildinfo, store_read_project, store_read_package, meta_exists, quote_plus, get_buildconfig, is_package_dir, dgst
+from .core import get_buildinfo, meta_exists, quote_plus, get_buildconfig, dgst
 from .core import get_binarylist, get_binary_file, run_external, return_external, raw_input
 from .fetch import Fetcher, OscFileGrabber, verify_pacs
 from .meter import create_text_meter
@@ -456,30 +458,21 @@ def get_built_files(pacdir, buildtype):
 
 
 def get_repo(path):
-    """Walks up path looking for any repodata directories.
+    """
+    Walks up path looking for any repodata directories.
 
     :param path: path to a directory
-    :return: path to repository directory containing repodata directory
+    :return: path to repository directory containing repodata directory with repomd.xml file
     :rtype: str
     """
-    oldDirectory = None
-    currentDirectory = os.path.abspath(path)
-    repositoryDirectory = None
 
-    # while there are still parent directories
-    while currentDirectory != oldDirectory:
-        children = os.listdir(currentDirectory)
+    for root, dirs, files in os.walk(path):
+        if not "repodata" in dirs:
+            continue
+        if "repomd.xml" in os.listdir(os.path.join(root, "repodata")):
+            return root
+    return None
 
-        if "repodata" in children:
-            repositoryDirectory = currentDirectory
-            break
-
-        # ascend
-        oldDirectory = currentDirectory
-        currentDirectory = os.path.abspath(os.path.join(oldDirectory,
-                                                        os.pardir))
-
-    return repositoryDirectory
 
 
 def get_prefer_pkgs(dirs, wanted_arch, type, cpio):
@@ -557,14 +550,24 @@ trustprompt = """Would you like to ...
 ? """
 
 
-def check_trusted_projects(apiurl, projects):
+def check_trusted_projects(apiurl, projects, interactive=True):
     trusted = conf.config['api_host_options'][apiurl]['trusted_prj']
     tlen = len(trusted)
     for prj in projects:
-        if prj not in trusted:
+        is_trusted = False
+        for pattern in trusted:
+            if fnmatch.fnmatch(prj, pattern):
+                is_trusted = True
+                break
+        if not is_trusted:
             print("\nThe build root needs packages from project '%s'." % prj)
             print("Note that malicious packages can compromise the build result or even your system.")
-            r = raw_input(trustprompt % {'project': prj})
+
+            if interactive:
+                r = raw_input(trustprompt % {'project': prj})
+            else:
+                r = "0"
+
             if r == '1':
                 print("adding '%s' to oscrc: ['%s']['trusted_prj']" % (prj, apiurl))
                 trusted.append(prj)
@@ -593,29 +596,45 @@ def get_kiwipath_from_buildinfo(bi, prj, repo):
     return kiwipath
 
 
-def calculate_prj_pac(opts, descr):
-    project = opts.alternative_project or store_read_project('.')
+def calculate_prj_pac(store, opts, descr):
+    project = opts.alternative_project or store.project
     if opts.local_package:
         package = os.path.splitext(os.path.basename(descr))[0]
     else:
-        package = store_read_package('.')
+        store.assert_is_package()
+        package = store.package
     return project, package
 
 
-def calculate_build_root(apihost, prj, pac, repo, arch):
-    buildroot = os.environ.get('OSC_BUILD_ROOT', conf.config['build-root']) \
-        % {'repo': repo, 'arch': arch, 'project': prj, 'package': pac, 'apihost': apihost}
+def calculate_build_root_user(vm_type):
+    if vm_type in ("kvm", "podman"):
+        return getpass.getuser()
+    return None
+
+
+def calculate_build_root(apihost, prj, pac, repo, arch, user=None):
+    user = user or ""
+    dash_user = f"-{user:s}" if user else ""
+    buildroot = conf.config["build-root"] % {
+        'apihost': apihost,
+        'project': prj,
+        'package': pac,
+        'repo': repo,
+        'arch': arch,
+        "user": user,
+        "dash_user": dash_user,
+    }
     return buildroot
 
 
 def build_as_user():
-    if os.environ.get('OSC_SU_WRAPPER', conf.config['su-wrapper']).split():
+    if conf.config["su-wrapper"]:
         return False
     return True
 
 
 def su_wrapper(cmd):
-    sucmd = os.environ.get('OSC_SU_WRAPPER', conf.config['su-wrapper']).split()
+    sucmd = conf.config['su-wrapper'].split()
     if sucmd:
         if sucmd[0] == 'su':
             if sucmd[-1] == '-c':
@@ -630,14 +649,19 @@ def run_build(opts, *args):
     cmd = [conf.config['build-cmd']]
     cmd += args
 
-    cmd = su_wrapper(cmd)
+    if opts.vm_type:
+        cmd.extend(["--vm-type", opts.vm_type])
+
+    user = calculate_build_root_user(opts.vm_type)
+    if not user:
+        cmd = su_wrapper(cmd)
 
     if not opts.userootforbuild:
         cmd.append('--norootforbuild')
     return run_external(cmd[0], *cmd[1:])
 
 
-def main(apiurl, opts, argv):
+def main(apiurl, store, opts, argv):
 
     repo = argv[0]
     arch = argv[1]
@@ -766,11 +790,11 @@ def main(apiurl, opts, argv):
         prj = opts.alternative_project
         pac = '_repository'
     else:
-        prj = store_read_project(os.curdir)
+        prj = store.project
         if opts.local_package:
             pac = '_repository'
         else:
-            pac = store_read_package(os.curdir)
+            pac = store.package
     if opts.multibuild_package:
         buildargs.append('--buildflavor=%s' % opts.multibuild_package)
         pac = pac + ":" + opts.multibuild_package
@@ -779,38 +803,19 @@ def main(apiurl, opts, argv):
     if opts.wipe:
         buildargs.append("--wipe")
 
-    orig_build_root = config['build-root']
-    # make it possible to override configuration of the rc file
-    for var in ['OSC_PACKAGECACHEDIR', 'OSC_SU_WRAPPER', 'OSC_BUILD_ROOT']:
-        val = os.getenv(var)
-        if val:
-            if var.startswith('OSC_'):
-                var = var[4:]
-            var = var.lower().replace('_', '-')
-            if var in config:
-                print('Overriding config value for %s=\'%s\' with \'%s\'' % (var, config[var], val))
-            config[var] = val
-
     pacname = pac
     if pacname == '_repository':
         if not opts.local_package:
             try:
-                pacname = store_read_package(os.curdir)
+                pacname = store.package
             except oscerr.NoWorkingCopy:
                 opts.local_package = True
         if opts.local_package:
             pacname = os.path.splitext(os.path.basename(build_descr))[0]
     apihost = urlsplit(apiurl)[1]
     if not build_root:
-        build_root = config['build-root']
-        if build_root == orig_build_root:
-            # ENV var was not set
-            build_root = config['api_host_options'][apiurl].get('build-root', build_root)
-        try:
-            build_root = build_root % {'repo': repo, 'arch': arch,
-                                       'project': prj, 'package': pacname, 'apihost': apihost}
-        except KeyError:
-            pass
+        user = calculate_build_root_user(vm_type)
+        build_root = calculate_build_root(apihost, prj, pacname, repo, arch, user)
 
     # We configure sccache after pacname, so that in default cases we can have an sccache for each
     # package to prevent cross-cache polutions. It helps to make the local-use case a bit nicer.
@@ -832,7 +837,7 @@ def main(apiurl, opts, argv):
     bc_file = None
     bi_filename = '_buildinfo-%s-%s.xml' % (repo, arch)
     bc_filename = '_buildconfig-%s-%s' % (repo, arch)
-    if is_package_dir('.') and os.access(core.store, os.W_OK):
+    if store.is_package and os.access(core.store, os.W_OK):
         bi_filename = os.path.join(os.getcwd(), core.store, bi_filename)
         bc_filename = os.path.join(os.getcwd(), core.store, bc_filename)
     elif not os.access('.', os.W_OK):
@@ -857,11 +862,11 @@ def main(apiurl, opts, argv):
     if opts.noinit:
         buildargs.append('--noinit')
 
-    if not is_package_dir('.'):
-        opts.noservice = True
+    if not store.is_package:
+        opts.skip_local_service_run = True
 
     # check for source services
-    if not opts.offline and not opts.noservice:
+    if not opts.offline and not opts.skip_local_service_run:
         p = core.Package(os.curdir)
         r = p.run_source_services(verbose=True)
         if r:
@@ -1057,6 +1062,9 @@ def main(apiurl, opts, argv):
     if opts.build_opt:
         buildargs += opts.build_opt
 
+    if opts.buildtool_opt:
+        buildargs += [f"--buildtool-opt={opt}" for opt in opts.buildtool_opt]
+
     # real arch of this machine
     # vs.
     # arch we are supposed to build for
@@ -1263,10 +1271,24 @@ def main(apiurl, opts, argv):
                     if name == filename:
                         print("Using prefered package: " + path + "/" + filename)
                         os.unlink(tffn)
-                        if opts.linksources:
-                            os.link(path + "/" + filename, tffn)
-                        else:
-                            os.symlink(path + "/" + filename, tffn)
+
+        if prefer_pkgs:
+            localpkgdir = "repos/_local/"
+            os.mkdir(localpkgdir)
+            buildargs.append("--kiwi-parameter")
+            buildargs.append("--add-repo")
+            buildargs.append("--kiwi-parameter")
+            buildargs.append(f"dir://./{localpkgdir}")
+            buildargs.append("--kiwi-parameter")
+            buildargs.append("--add-repotype")
+            buildargs.append("--kiwi-parameter")
+            buildargs.append("rpm-md")
+            for name, path in prefer_pkgs.items():
+                tffn = os.path.join(localpkgdir, os.path.basename(path))
+                if opts.linksources:
+                    os.link(path, tffn)
+                else:
+                    os.symlink(path, tffn)
 
     if build_type == 'kiwi':
         # Is a obsrepositories tag used?
@@ -1362,7 +1384,7 @@ def main(apiurl, opts, argv):
 
     for i in bi.deps:
         if i.hdrmd5:
-            if not i.name.startswith('container:') and i.pacsuffix != 'rpm':
+            if not i.name.startswith('container:') and not i.fullfilename.endswith(".rpm"):
                 continue
             if i.name.startswith('container:'):
                 hdrmd5 = dgst(i.fullfilename)
@@ -1431,59 +1453,69 @@ def main(apiurl, opts, argv):
         else:
             my_build_swap = build_root + '/swap'
 
-        vm_options = ['--vm-type=%s' % vm_type]
+        vm_options = [f"--vm-type={vm_type}"]
         if vm_telnet:
-            vm_options += ['--vm-telnet=' + vm_telnet]
+            vm_options += [f"--vm-telnet={vm_telnet}"]
         if vm_memory:
-            vm_options += ['--memory=' + vm_memory]
+            vm_options += [f"--memory={vm_memory}"]
         if vm_type != 'lxc' and vm_type != 'nspawn':
-            vm_options += ['--vm-disk=' + my_build_device]
-            vm_options += ['--vm-swap=' + my_build_swap]
-            vm_options += ['--logfile=%s/.build.log' % build_root]
+            vm_options += [f"--vm-disk={my_build_device}"]
+            vm_options += [f"--vm-swap={my_build_swap}"]
+            vm_options += [f"--logfile={build_root}/.build.log"]
             if vm_type == 'kvm':
                 if config['build-kernel']:
-                    vm_options += ['--vm-kernel=' + config['build-kernel']]
+                    vm_options += [f"--vm-kernel={config['build-kernel']}"]
                 if config['build-initrd']:
-                    vm_options += ['--vm-initrd=' + config['build-initrd']]
+                    vm_options += [f"--vm-initrd={config['build-initrd']}"]
 
             build_root += '/.mount'
         if vm_disk_size:
-            vm_options += ['--vmdisk-rootsize=' + vm_disk_size]
+            vm_options += [f"--vmdisk-rootsize={vm_disk_size}"]
 
         if config['build-vmdisk-swapsize']:
-            vm_options += ['--vmdisk-swapsize=' + config['build-vmdisk-swapsize']]
+            vm_options += [f"--vmdisk-swapsize={config['build-vmdisk-swapsize']}"]
         if config['build-vmdisk-filesystem']:
-            vm_options += ['--vmdisk-filesystem=' + config['build-vmdisk-filesystem']]
+            vm_options += [f"--vmdisk-filesystem={config['build-vmdisk-filesystem']}"]
         if config['build-vm-user']:
-            vm_options += ['--vm-user=' + config['build-vm-user']]
+            vm_options += [f"--vm-user={config['build-vm-user']}"]
 
     if opts.preload:
         print("Preload done for selected repo/arch.")
         sys.exit(0)
 
     print('Running build')
-    cmd = [config['build-cmd'], '--root=' + build_root,
-           '--rpmlist=' + rpmlist_filename,
-           '--dist=' + bc_filename,
-           '--arch=' + bi.buildarch]
+    cmd = [
+        config['build-cmd'],
+        f"--root={build_root}",
+        f"--rpmlist={rpmlist_filename}",
+        f"--dist={bc_filename}",
+        f"--arch={bi.buildarch}",
+    ]
     cmd += specialcmdopts + vm_options + buildargs
     cmd += [build_descr]
 
-    cmd = su_wrapper(cmd)
+    # determine if we're building under root (user == None) and use su_wrapper accordingly
+    if calculate_build_root_user(vm_type) is None:
+        cmd = su_wrapper(cmd)
 
     # change personality, if needed
     if hostarch != bi.buildarch and bi.buildarch in change_personality:
         cmd = [change_personality[bi.buildarch]] + cmd
 
     # record our settings for later builds
-    if is_package_dir(os.curdir):
-        core.store_write_last_buildroot(os.curdir, repo, arch, vm_type)
+    if store.is_package:
+        store.last_buildroot = repo, arch, vm_type
 
     try:
         rc = run_external(cmd[0], *cmd[1:])
         if rc:
             print()
-            print('The buildroot was:', build_root)
+            print(f"Build failed with exit code {rc}")
+            print(f"The buildroot was: {build_root}")
+            print()
+            print("Cleaning the build root may fix the problem or allow you to start debugging from a well-defined state:")
+            print("  - add '--clean' option to your 'osc build' command")
+            print("  - run 'osc wipe [--vm-type=...]' prior running your 'osc build' command again")
             sys.exit(rc)
     except KeyboardInterrupt as keyboard_interrupt_exception:
         print("keyboard interrupt, killing build ...")
