@@ -21,6 +21,7 @@ import urllib3.response
 import urllib3.util
 
 from . import __version__
+from . import _private
 from . import conf
 from . import oscerr
 from . import oscssl
@@ -104,10 +105,15 @@ def get_proxy_manager(env):
         proxy_url = f"{proxy_purl.scheme}://{proxy_purl.host}"
 
     proxy_headers = urllib3.make_headers(
-        proxy_basic_auth=proxy_purl.auth,
         user_agent=f"osc/{__version__}",
     )
 
+    if proxy_purl.auth:
+        proxy_basic_auth = urllib.parse.unquote(proxy_purl.auth)
+        proxy_basic_auth = proxy_basic_auth.encode("utf-8")
+        proxy_basic_auth = base64.b64encode(proxy_basic_auth).decode()
+        proxy_headers["Proxy-Authorization"] = f"Basic {proxy_basic_auth:s}"
+
     manager = urllib3.ProxyManager(proxy_url, proxy_headers=proxy_headers)
     return manager
 
@@ -135,12 +141,12 @@ def http_request_wrap_file(func):
     Turn file path into a file object and close it automatically
     by using a context manager.
     """
-    def new_func(method, url, headers=None, data=None, file=None):
+    def new_func(method, url, headers=None, data=None, file=None, retry_on_400: bool = True):
         if file:
             with open(file, "rb") as f:
-                return func(method, url, headers, data, file=f)
+                return func(method, url, headers, data, f, retry_on_400)
         else:
-            return func(method, url, headers, data, file)
+            return func(method, url, headers, data, file, retry_on_400)
 
     new_func.__name__ = func.__name__
     new_func.__doc__ = func.__doc__
@@ -148,7 +154,7 @@ def http_request_wrap_file(func):
 
 
 @http_request_wrap_file
-def http_request(method: str, url: str, headers=None, data=None, file=None):
+def http_request(method: str, url: str, headers=None, data=None, file=None, retry_on_400: bool = True):
     """
     Send a HTTP request to a server.
 
@@ -168,6 +174,7 @@ def http_request(method: str, url: str, headers=None, data=None, file=None):
     :param headers: Dictionary of custom headers to send.
     :param data: Data to send in the request body (conflicts with `file`).
     :param file: Path to a file to send as data in the request body (conflicts with `data`).
+    :param retry_on_400: Whether to retry on receiving HTTP status code 400.
     """
 
     purl = urllib3.util.parse_url(url)
@@ -227,18 +234,46 @@ def http_request(method: str, url: str, headers=None, data=None, file=None):
         else:
             retries_kwargs = {"method_whitelist": None}
 
+
+        status_forcelist = (
+            500,  # Internal Server Error
+            502,  # Bad Gateway
+            503,  # Service Unavailable
+            504,  # Gateway Timeout
+        )
+        if retry_on_400:
+            status_forcelist = (
+                400,  # Bad Request; retry on 400: service in progress
+            ) + status_forcelist
+
         pool_kwargs["retries"] = urllib3.Retry(
             total=int(conf.config["http_retries"]),
             backoff_factor=2,
+            status_forcelist=status_forcelist,
+            # don't raise because we want an actual response rather than a MaxRetryError with "too many <status_code> error responses" message
+            raise_on_status=False,
             **retries_kwargs,
         )
 
         if purl.scheme == "https":
             ssl_context = oscssl.create_ssl_context()
             ssl_context.load_default_certs()
-            # turn cert verification off if sslcertck = 0
-            pool_kwargs["cert_reqs"] = "CERT_REQUIRED" if options["sslcertck"] else "CERT_NONE"
             pool_kwargs["ssl_context"] = ssl_context
+            # turn cert verification off if sslcertck = 0
+
+            if options["cafile"] or options["capath"]:
+                ssl_context.load_verify_locations(cafile=options["cafile"], capath=options["capath"])
+
+            # urllib3 v1
+            pool_kwargs["cert_reqs"] = "CERT_REQUIRED" if options["sslcertck"] else "CERT_NONE"
+
+            # urllib3 v2
+            if options["sslcertck"]:
+                ssl_context.check_hostname = True
+                ssl_context.verify_mode = ssl.CERT_REQUIRED
+            else:
+                ssl_context.check_hostname = False
+                ssl_context.verify_mode = ssl.CERT_NONE
 
         if purl.scheme == "http" and HTTP_PROXY_MANAGER and not urllib.request.proxy_bypass(url):
             # connection through HTTP proxy
@@ -288,9 +323,10 @@ def http_request(method: str, url: str, headers=None, data=None, file=None):
     # https://github.com/openSUSE/open-build-service/pull/13019
     headers.add("Accept", "application/xml")
 
-    if data or file:
-        # osc/obs data is usually XML
+    if method == "PUT" or (method == "POST" and (data or file)):
         headers.add("Content-Type", "application/xml; charset=utf-8")
+    elif method == "POST":
+        headers.add("Content-Type", "application/x-www-form-urlencoded")
 
     if purl.scheme == "http" and HTTP_PROXY_MANAGER:
         # HTTP proxy requires full URL with 'same host' checking off
@@ -522,7 +558,12 @@ class BasicAuthHandler(AuthHandlerBase):
             return False
         if not self.user or not self.password:
             return False
-        request_headers.update(urllib3.make_headers(basic_auth=f"{self.user}:{self.password}"))
+
+        basic_auth = f"{self.user:s}:{self.password:s}"
+        basic_auth = basic_auth.encode("utf-8")
+        basic_auth = base64.b64encode(basic_auth).decode()
+        request_headers["Authorization"] = f"Basic {basic_auth:s}"
+
         return True
 
     def process_response(self, url, request_headers, response):
@@ -649,9 +690,7 @@ class SignatureAuthHandler(AuthHandlerBase):
             return False
 
         if not self.ssh_keygen_path:
-            if conf.config["debug"]:
-                msg = "Skipping signature auth because ssh-keygen is not available"
-                print(msg, file=sys.stderr)
+            _private.print_msg("Skipping signature auth because ssh-keygen is not available", print_to="debug")
             return False
 
         if not self.sshkey_known():
