@@ -4,103 +4,46 @@
 # either version 2, or (at your option) any later version.
 
 
+import os
+import re
+import shutil
+import subprocess
+import sys
+import tempfile
+from urllib.parse import quote_plus
+from urllib.request import HTTPError
 
-import sys, os
-
-try:
-    from urllib.parse import quote_plus
-    from urllib.request import HTTPBasicAuthHandler, HTTPCookieProcessor, HTTPPasswordMgrWithDefaultRealm, HTTPError
-except ImportError:
-    #python 2.x
-    from urllib.parse import quote_plus
-    from urllib.request import HTTPBasicAuthHandler, HTTPCookieProcessor, HTTPPasswordMgrWithDefaultRealm
-    from urllib.error import HTTPError
-
-from urlgrabber.grabber import URLGrabber, URLGrabError
-from urlgrabber.mirror import MirrorGroup
-from .core import makeurl, streamfile
-from .util import packagequery, cpio
+from . import checker as osc_checker
 from . import conf
 from . import oscerr
-import tempfile
-import re
-try:
-    from .meter import TextMeter
-except:
-    TextMeter = None
-
-
-def join_url(self, base_url, rel_url):
-    """to override _join_url of MirrorGroup, because we want to
-    pass full URLs instead of base URL where relative_url is added later...
-    IOW, we make MirrorGroup ignore relative_url
-    """
-    return base_url
-
-
-class OscFileGrabber(URLGrabber):
-    def __init__(self, progress_obj=None):
-        # we cannot use super because we still have to support
-        # older urlgrabber versions where URLGrabber is an old-style class
-        URLGrabber.__init__(self)
-        self.progress_obj = progress_obj
-
-    def urlgrab(self, url, filename, text=None, **kwargs):
-        if url.startswith('file://'):
-            f = url.replace('file://', '', 1)
-            if os.path.isfile(f):
-                return f
-            else:
-                raise URLGrabError(2, 'Local file \'%s\' does not exist' % f)
-        with file(filename, 'wb') as f:
-            try:
-                for i in streamfile(url, progress_obj=self.progress_obj,
-                                    text=text):
-                    f.write(i)
-            except HTTPError as e:
-                exc = URLGrabError(14, str(e))
-                exc.url = url
-                exc.exception = e
-                exc.code = e.code
-                raise exc
-            except IOError as e:
-                raise URLGrabError(4, str(e))
-        return filename
+from .core import makeurl, dgst
+from .grabber import OscFileGrabber, OscMirrorGroup
+from .meter import create_text_meter
+from .util import packagequery, cpio
+from .util.helper import decode_it
 
 
 class Fetcher:
-    def __init__(self, cachedir='/tmp', api_host_options={}, urllist=[],
-            http_debug=False, cookiejar=None, offline=False, enable_cpio=True):
+    def __init__(self, cachedir='/tmp', urllist=None,
+                 http_debug=False, cookiejar=None, offline=False,
+                 enable_cpio=True, modules=None, download_api_only=False):
         # set up progress bar callback
-        if sys.stdout.isatty() and TextMeter:
-            self.progress_obj = TextMeter(fo=sys.stdout)
-        else:
-            self.progress_obj = None
+        self.progress_obj = None
+        if sys.stdout.isatty():
+            self.progress_obj = create_text_meter(use_pb_fallback=False)
 
         self.cachedir = cachedir
-        self.urllist = urllist
+        # generic download URL lists
+        self.urllist = urllist or []
+        self.modules = modules or []
         self.http_debug = http_debug
         self.offline = offline
         self.cpio = {}
         self.enable_cpio = enable_cpio
+        self.download_api_only = download_api_only
 
-        passmgr = HTTPPasswordMgrWithDefaultRealm()
-        for host in api_host_options:
-            passmgr.add_password(None, host, api_host_options[host]['user'],
-                                 api_host_options[host]['pass'])
-        openers = (HTTPBasicAuthHandler(passmgr), )
-        if cookiejar:
-            openers += (HTTPCookieProcessor(cookiejar), )
         self.gr = OscFileGrabber(progress_obj=self.progress_obj)
 
-    def failureReport(self, errobj):
-        """failure output for failovers from urlgrabber"""
-        if errobj.url.startswith('file://'):
-            return {}
-        print('Trying openSUSE Build Service server for %s (%s), not found at %s.'
-              % (self.curpac, self.curpac.project, errobj.url.split('/')[2]))
-        return {}
-
     def __add_cpio(self, pac):
         prpap = '%s/%s/%s/%s' % (pac.project, pac.repository, pac.repoarch, pac.repopackage)
         self.cpio.setdefault(prpap, {})[pac.repofilename] = pac
@@ -110,6 +53,8 @@ class Fetcher:
             return
         query = ['binary=%s' % quote_plus(i) for i in pkgs]
         query.append('view=cpio')
+        for module in self.modules:
+            query.append('module=' + module)
         try:
             url = makeurl(apiurl, ['build', project, repo, arch, package], query=query)
             sys.stdout.write("preparing download ...\r")
@@ -124,52 +69,60 @@ class Fetcher:
                     # getbinarylist instead of the public/... route
                     # (which is routed to getbinaries)
                     # getbinaries does not support kiwi builds
-                    if hdr.filename == '.errors':
+                    if hdr.filename == b'.errors':
                         archive.copyin_file(hdr.filename)
                         raise oscerr.APIError('CPIO archive is incomplete '
                                               '(see .errors file)')
                     if package == '_repository':
-                        n = re.sub(r'\.pkg\.tar\..z$', '.arch', hdr.filename)
-                        pac = pkgs[n.rsplit('.', 1)[0]]
+                        n = re.sub(br'\.pkg\.tar\.(zst|.z)$', b'.arch', hdr.filename)
+                        if n.startswith(b'container:'):
+                            n = re.sub(br'\.tar\.(zst|.z)$', b'.tar', hdr.filename)
+                            pac = pkgs[decode_it(n.rsplit(b'.', 1)[0])]
+                            pac.canonname = hdr.filename
+                        else:
+                            pac = pkgs[decode_it(n.rsplit(b'.', 1)[0])]
                     else:
                         # this is a kiwi product
-                        pac = pkgs[hdr.filename]
+                        pac = pkgs[decode_it(hdr.filename)]
 
                     # Extract a single file from the cpio archive
+                    fd = None
+                    tmpfile = None
                     try:
                         fd, tmpfile = tempfile.mkstemp(prefix='osc_build_file')
                         archive.copyin_file(hdr.filename,
-                                            os.path.dirname(tmpfile),
-                                            os.path.basename(tmpfile))
+                                            decode_it(os.path.dirname(tmpfile)),
+                                            decode_it(os.path.basename(tmpfile)))
                         self.move_package(tmpfile, pac.localdir, pac)
                     finally:
-                        os.close(fd)
-                        if os.path.exists(tmpfile):
+                        if fd is not None:
+                            os.close(fd)
+                        if tmpfile is not None and os.path.exists(tmpfile):
                             os.unlink(tmpfile)
 
-                for pac in list(pkgs.values()):
+                for pac in pkgs.values():
                     if not os.path.isfile(pac.fullfilename):
                         raise oscerr.APIError('failed to fetch file \'%s\': '
                                               'missing in CPIO archive' %
                                               pac.repofilename)
-        except URLGrabError as e:
-            if e.errno != 14 or e.code != 414:
+        except HTTPError as e:
+            if e.code != 414:
                 raise
             # query str was too large
             keys = list(pkgs.keys())
             if len(keys) == 1:
                 raise oscerr.APIError('unable to fetch cpio archive: '
                                       'server always returns code 414')
-            n = len(pkgs) / 2
-            new_pkgs = dict([(k, pkgs[k]) for k in keys[:n]])
+            n = int(len(pkgs) / 2)
+            new_pkgs = {k: pkgs[k] for k in keys[:n]}
             self.__download_cpio_archive(apiurl, project, repo, arch,
                                          package, **new_pkgs)
-            new_pkgs = dict([(k, pkgs[k]) for k in keys[n:]])
+            new_pkgs = {k: pkgs[k] for k in keys[n:]}
             self.__download_cpio_archive(apiurl, project, repo, arch,
                                          package, **new_pkgs)
 
     def __fetch_cpio(self, apiurl):
-        for prpap, pkgs in list(self.cpio.items()):
+        for prpap, pkgs in self.cpio.items():
             project, repo, arch, package = prpap.split('/', 3)
             self.__download_cpio_archive(apiurl, project, repo, arch, package, **pkgs)
 
@@ -177,8 +130,7 @@ class Fetcher:
         # for use by the failure callback
         self.curpac = pac
 
-        MirrorGroup._join_url = join_url
-        mg = MirrorGroup(self.gr, pac.urllist, failure_callback=(self.failureReport, (), {}))
+        mg = OscMirrorGroup(self.gr, pac.urllist)
 
         if self.http_debug:
             print('\nURLs to try for package \'%s\':' % pac, file=sys.stderr)
@@ -188,35 +140,44 @@ class Fetcher:
         try:
             with tempfile.NamedTemporaryFile(prefix='osc_build',
                                              delete=False) as tmpfile:
-                mg.urlgrab(pac.filename, filename=tmpfile.name,
-                           text='%s(%s) %s' % (prefix, pac.project, pac.filename))
-                self.move_package(tmpfile.name, pac.localdir, pac)
-        except URLGrabError as e:
-            if self.enable_cpio and e.errno == 256:
-                self.__add_cpio(pac)
-                return
-            print()
-            print('Error:', e.strerror, file=sys.stderr)
-            print('Failed to retrieve %s from the following locations '
-                  '(in order):' % pac.filename, file=sys.stderr)
-            print('\n'.join(pac.urllist), file=sys.stderr)
-            sys.exit(1)
+                mg_stat = mg.urlgrab(pac.filename, filename=tmpfile.name,
+                                     text='%s(%s) %s' % (prefix, pac.project, pac.filename))
+                if mg_stat:
+                    self.move_package(tmpfile.name, pac.localdir, pac)
+
+            if not mg_stat:
+                if self.enable_cpio:
+                    print('%s/%s: attempting download from api, since not found'
+                          % (pac.project, pac.name))
+                    self.__add_cpio(pac)
+                    return
+                print()
+                print('Error: Failed to retrieve %s from the following locations '
+                      '(in order):' % pac.filename, file=sys.stderr)
+                print('\n'.join(pac.urllist), file=sys.stderr)
+                sys.exit(1)
         finally:
             if os.path.exists(tmpfile.name):
                 os.unlink(tmpfile.name)
 
     def move_package(self, tmpfile, destdir, pac_obj=None):
-        import shutil
-        pkgq = packagequery.PackageQuery.query(tmpfile, extra_rpmtags=(1044, 1051, 1052))
-        if pkgq:
-            canonname = pkgq.canonname()
-        else:
-            if pac_obj is None:
-                print('Unsupported file type: ', tmpfile, file=sys.stderr)
-                sys.exit(1)
-            canonname = pac_obj.binary
+        canonname = None
+        if pac_obj and pac_obj.name.startswith('container:'):
+            canonname = pac_obj.canonname
+        if canonname is None:
+            pkgq = packagequery.PackageQuery.query(tmpfile, extra_rpmtags=(1044, 1051, 1052))
+            if pkgq:
+                canonname = pkgq.canonname()
+            else:
+                if pac_obj is None:
+                    print('Unsupported file type: ', tmpfile, file=sys.stderr)
+                    sys.exit(1)
+                canonname = pac_obj.binary
+        decoded_canonname = decode_it(canonname)
+        if b'/' in canonname or '/' in decoded_canonname:
+            raise oscerr.OscIOError(None, 'canonname contains a slash')
 
-        fullfilename = os.path.join(destdir, canonname)
+        fullfilename = os.path.join(destdir, decoded_canonname)
         if pac_obj is not None:
             pac_obj.canonname = canonname
             pac_obj.fullfilename = fullfilename
@@ -233,19 +194,55 @@ class Fetcher:
                 print(e, file=sys.stderr)
                 sys.exit(1)
 
+    def _build_urllist(self, buildinfo, pac):
+        if self.download_api_only:
+            return []
+        urllist = self.urllist
+        key = '%s/%s' % (pac.project, pac.repository)
+        project_repo_url = buildinfo.urls.get(key)
+        if project_repo_url is not None:
+            urllist = [project_repo_url]
+        return urllist
+
     def run(self, buildinfo):
+        apiurl = buildinfo.apiurl
         cached = 0
         all = len(buildinfo.deps)
         for i in buildinfo.deps:
-            i.makeurls(self.cachedir, self.urllist)
+            urllist = self._build_urllist(buildinfo, i)
+            i.makeurls(self.cachedir, urllist)
+            # find container extension by looking in the cache
+            if i.name.startswith('container:') and i.fullfilename.endswith('.tar.xz'):
+                for ext in ['.tar.xz', '.tar.gz', '.tar']:
+                    if os.path.exists(i.fullfilename[:-7] + ext):
+                        i.canonname = i.canonname[:-7] + ext
+                        i.makeurls(self.cachedir, urllist)
+
             if os.path.exists(i.fullfilename):
                 cached += 1
+                if not i.name.startswith('container:') and i.pacsuffix != 'rpm':
+                    continue
+
+                hdrmd5_is_valid = True
                 if i.hdrmd5:
-                    from .util import packagequery
-                    hdrmd5 = packagequery.PackageQuery.queryhdrmd5(i.fullfilename)
-                    if not hdrmd5 or hdrmd5 != i.hdrmd5:
+                    if i.name.startswith('container:'):
+                        hdrmd5 = dgst(i.fullfilename)
+                        if hdrmd5 != i.hdrmd5:
+                            hdrmd5_is_valid = False
+                    else:
+                        hdrmd5 = packagequery.PackageQuery.queryhdrmd5(i.fullfilename)
+                        if hdrmd5 != i.hdrmd5:
+                            if conf.config["api_host_options"][apiurl]["disable_hdrmd5_check"]:
+                                print(f"Warning: Ignoring a hdrmd5 mismatch for {i.fullfilename}: {hdrmd5} (actual) != {i.hdrmd5} (expected)")
+                                hdrmd5_is_valid = True
+                            else:
+                                print(f"The file will be redownloaded from the API due to a hdrmd5 mismatch for {i.fullfilename}: {hdrmd5} (actual) != {i.hdrmd5} (expected)")
+                                hdrmd5_is_valid = False
+
+                    if not hdrmd5_is_valid:
                         os.unlink(i.fullfilename)
                         cached -= 1
+
         miss = 0
         needed = all - cached
         if all:
@@ -253,7 +250,6 @@ class Fetcher:
         print("%.1f%% cache miss. %d/%d dependencies cached.\n" % (miss, cached, all))
         done = 1
         for i in buildinfo.deps:
-            i.makeurls(self.cachedir, self.urllist)
             if not os.path.exists(i.fullfilename):
                 if self.offline:
                     raise oscerr.OscIOError(None,
@@ -261,18 +257,28 @@ class Fetcher:
                                             '--offline not possible.' %
                                             i.fullfilename)
                 self.dirSetup(i)
-                if i.hdrmd5 and self.enable_cpio:
-                    self.__add_cpio(i)
-                    done += 1
-                    continue
                 try:
                     # if there isn't a progress bar, there is no output at all
+                    prefix = ''
                     if not self.progress_obj:
                         print('%d/%d (%s) %s' % (done, needed, i.project, i.filename))
-                    self.fetch(i)
-                    if self.progress_obj:
-                        print("  %d/%d\r" % (done, needed), end=' ')
-                        sys.stdout.flush()
+                    else:
+                        prefix = '[%d/%d] ' % (done, needed)
+                    self.fetch(i, prefix=prefix)
+
+                    if not os.path.isfile(i.fullfilename):
+                        # if the file wasn't downloaded and cannot be found on disk,
+                        # mark it for downloading from the API
+                        self.__add_cpio(i)
+                    else:
+                        hdrmd5 = packagequery.PackageQuery.queryhdrmd5(i.fullfilename)
+                        if hdrmd5 != i.hdrmd5:
+                            if conf.config["api_host_options"][apiurl]["disable_hdrmd5_check"]:
+                                print(f"Warning: Ignoring a hdrmd5 mismatch for {i.fullfilename}: {hdrmd5} (actual) != {i.hdrmd5} (expected)")
+                            else:
+                                print(f"The file will be redownloaded from the API due to a hdrmd5 mismatch for {i.fullfilename}: {hdrmd5} (actual) != {i.hdrmd5} (expected)")
+                                os.unlink(i.fullfilename)
+                                self.__add_cpio(i)
 
                 except KeyboardInterrupt:
                     print('Cancelled by user (ctrl-c)')
@@ -290,14 +296,15 @@ class Fetcher:
             dest += '/_pubkey'
 
             url = makeurl(buildinfo.apiurl, ['source', i, '_pubkey'])
+            try_parent = False
             try:
                 if self.offline and not os.path.exists(dest):
                     # may need to try parent
-                    raise URLGrabError(2)
+                    try_parent = True
                 elif not self.offline:
                     OscFileGrabber().urlgrab(url, dest)
                 # not that many keys usually
-                if i not in buildinfo.prjkeys:
+                if i not in buildinfo.prjkeys and not try_parent:
                     buildinfo.keys.append(dest)
                     buildinfo.prjkeys.append(i)
             except KeyboardInterrupt:
@@ -306,14 +313,16 @@ class Fetcher:
                 if os.path.exists(dest):
                     os.unlink(dest)
                 sys.exit(0)
-            except URLGrabError as e:
+            except HTTPError as e:
                 # Not found is okay, let's go to the next project
-                if e.errno == 14 and e.code != 404:
+                if e.code != 404:
                     print("Invalid answer from server", e, file=sys.stderr)
                     sys.exit(1)
+                try_parent = True
 
+            if try_parent:
                 if self.http_debug:
-                    print("can't fetch key for %s: %s" % (i, e.strerror), file=sys.stderr)
+                    print("can't fetch key for %s" % (i), file=sys.stderr)
                     print("url: %s" % url, file=sys.stderr)
 
                 if os.path.exists(dest):
@@ -333,8 +342,6 @@ def verify_pacs_old(pac_list):
        Check all packages in one go, since this takes only 6 seconds on my Athlon 700
        instead of 20 when calling 'rpm -K' for each of them.
        """
-    import subprocess
-
     if not pac_list:
         return
 
@@ -383,7 +390,7 @@ def verify_pacs_old(pac_list):
 
 - You may use --no-verify to skip the verification (which is a risk for your system).
 """ % {'name': missing_key,
-       'dir': os.path.expanduser('~')}, file=sys.stderr)
+                    'dir': os.path.expanduser('~')}, file=sys.stderr)
 
             else:
                 print("""
@@ -412,9 +419,8 @@ def verify_pacs(bi):
 
     print("using keys from", ', '.join(bi.prjkeys))
 
-    from . import checker
     failed = False
-    checker = checker.Checker()
+    checker = osc_checker.Checker()
     try:
         checker.readkeys(bi.keys)
         for pkg in pac_list:
