@@ -1,57 +1,55 @@
-import osc.commandline
-import osc.core
-import osc.oscerr
 import os
 import re
 import sys
-from common import GET, POST, OscTestCase, addExpectedRequest, EXPECTED_REQUESTS
+import unittest
+
+import osc.commandline
+import osc.core
+import osc.oscerr
+
+from .common import GET, POST, OscTestCase, EXPECTED_REQUESTS
 
 
-FIXTURES_DIR = os.path.join(os.getcwd(), 'prdiff_fixtures')
-API_URL  = 'http://localhost/'
+FIXTURES_DIR = os.path.join(os.path.dirname(__file__), 'prdiff_fixtures')
 UPSTREAM = 'some:project'
-BRANCH   = 'home:user:branches:' + UPSTREAM
+BRANCH = 'home:user:branches:' + UPSTREAM
+
 
 def rdiff_url(pkg, oldprj, newprj):
-    return API_URL + 'source/%s/%s?unified=1&opackage=%s&oproject=%s&cmd=diff&expand=1&tarlimit=0&filelimit=0' % \
+    return 'http://localhost/source/%s/%s?unified=1&opackage=%s&oproject=%s&cmd=diff&expand=1&tarlimit=0&filelimit=0' % \
         (newprj, pkg, pkg, oldprj.replace(':', '%3A'))
 
+
 def request_url(prj):
-    return API_URL + 'search/request?match=%%28state%%2F%%40name%%3D%%27new%%27+or+state%%2F%%40name%%3D%%27review%%27%%29+and+%%28action%%2Ftarget%%2F%%40project%%3D%%27%s%%27+or+action%%2Fsource%%2F%%40project%%3D%%27%s%%27%%29' % \
-        tuple([prj.replace(':', '%3A')] * 2)
+    return "http://localhost/request" + f"?view=collection&project={prj}&states=new,review".replace(":", "%3A").replace(",", "%2C")
+
 
 def GET_PROJECT_PACKAGES(*projects):
     def decorator(test_method):
-        def wrapped_test_method(*args):
-            for project in projects:
-                addExpectedRequest('GET', API_URL + 'source/' + project,
-                                   file='%s/directory' % project)
-            test_method(*args)
-        # "rename" method otherwise we cannot specify a TestCaseClass.testName
-        # cmdline arg when using unittest.main()
-        wrapped_test_method.__name__ = test_method.__name__
-        return wrapped_test_method
+        # decorators get applied in the reversed order (bottom-up)
+        for project in reversed(projects):
+            test_method = GET(f'http://localhost/source/{project}', file=f'{project}/directory')(test_method)
+        return test_method
     return decorator
 
+
 def POST_RDIFF(oldprj, newprj):
     def decorator(test_method):
-        def wrapped_test_method(*args):
-            addExpectedRequest('POST', rdiff_url('common-one',   oldprj, newprj), exp='', text='')
-            addExpectedRequest('POST', rdiff_url('common-two',   oldprj, newprj), exp='', file='common-two-diff')
-            addExpectedRequest('POST', rdiff_url('common-three', oldprj, newprj), exp='', text='')
-            test_method(*args)
-        # "rename" method otherwise we cannot specify a TestCaseClass.testName
-        # cmdline arg when using unittest.main()
-        wrapped_test_method.__name__ = test_method.__name__
-        return wrapped_test_method
+        # decorators get applied in the reversed order (bottom-up)
+        test_method = POST(rdiff_url('common-three', oldprj, newprj), exp='', text='')(test_method)
+        test_method = POST(rdiff_url('common-two', oldprj, newprj), exp='', file='common-two-diff')(test_method)
+        test_method = POST(rdiff_url('common-one', oldprj, newprj), exp='', text='')(test_method)
+        return test_method
     return decorator
 
+
 def suite():
-    import unittest
-    return unittest.makeSuite(TestProjectDiff)
+    return unittest.defaultTestLoader.loadTestsFromTestCase(TestProjectDiff)
+
 
 class TestProjectDiff(OscTestCase):
     diff_hdr = 'Index: %s\n==================================================================='
+
     def _get_fixtures_dir(self):
         return FIXTURES_DIR
 
@@ -61,18 +59,16 @@ class TestProjectDiff(OscTestCase):
     def _run_prdiff(self, *args):
         """Runs osc prdiff, returning captured STDOUT as a string."""
         cli = osc.commandline.Osc()
-        argv = ['osc', '--no-keyring', '--no-gnome-keyring', 'prdiff']
+        argv = ['osc', '--no-keyring', 'prdiff']
         argv.extend(args)
         cli.main(argv=argv)
         return sys.stdout.getvalue()
 
-
     def testPrdiffTooManyArgs(self):
         def runner():
             self._run_prdiff('one', 'two', 'superfluous-arg')
         self.assertRaises(osc.oscerr.WrongArgs, runner)
 
-
     @GET_PROJECT_PACKAGES(UPSTREAM, BRANCH)
     @POST_RDIFF(UPSTREAM, BRANCH)
     @POST(rdiff_url('only-in-new', UPSTREAM, BRANCH), exp='', text='')
@@ -82,6 +78,7 @@ differs:   common-two
 identical: common-three
 identical: only-in-new
 """
+
         def runner():
             self._run_prdiff()
 
@@ -95,7 +92,6 @@ identical: only-in-new
         out = self._run_prdiff()
         self.assertEqualMultiline(out, exp)
 
-
     @GET_PROJECT_PACKAGES(UPSTREAM, BRANCH)
     @POST_RDIFF(UPSTREAM, BRANCH)
     @POST(rdiff_url('only-in-new', UPSTREAM, BRANCH), exp='', text='')
@@ -109,7 +105,6 @@ identical: only-in-new
         out = self._run_prdiff('home:user:branches:some:project')
         self.assertEqualMultiline(out, exp)
 
-
     @GET_PROJECT_PACKAGES('old:prj', 'new:prj')
     @POST_RDIFF('old:prj', 'new:prj')
     def testPrdiffTwoArgs(self):
@@ -121,7 +116,6 @@ identical: common-three
         out = self._run_prdiff('old:prj', 'new:prj')
         self.assertEqualMultiline(out, exp)
 
-
     @GET_PROJECT_PACKAGES('old:prj', 'new:prj')
     @POST_RDIFF('old:prj', 'new:prj')
     def testPrdiffOldOnly(self):
@@ -134,7 +128,6 @@ old only:  only-in-old
         out = self._run_prdiff('--show-not-in-new', 'old:prj', 'new:prj')
         self.assertEqualMultiline(out, exp)
 
-
     @GET_PROJECT_PACKAGES('old:prj', 'new:prj')
     @POST_RDIFF('old:prj', 'new:prj')
     def testPrdiffNewOnly(self):
@@ -147,7 +140,6 @@ new only:  only-in-new
         out = self._run_prdiff('--show-not-in-old', 'old:prj', 'new:prj')
         self.assertEqualMultiline(out, exp)
 
-
     @GET_PROJECT_PACKAGES('old:prj', 'new:prj')
     @POST_RDIFF('old:prj', 'new:prj')
     def testPrdiffDiffstat(self):
@@ -163,7 +155,6 @@ identical: common-three
         out = self._run_prdiff('--diffstat', 'old:prj', 'new:prj')
         self.assertEqualMultiline(out, exp)
 
-
     @GET_PROJECT_PACKAGES('old:prj', 'new:prj')
     @POST_RDIFF('old:prj', 'new:prj')
     def testPrdiffUnified(self):
@@ -187,9 +178,8 @@ identical: common-three
         out = self._run_prdiff('--unified', 'old:prj', 'new:prj')
         self.assertEqualMultiline(out, exp)
 
-
     @GET_PROJECT_PACKAGES('old:prj', 'new:prj')
-    @POST(rdiff_url('common-two',   'old:prj', 'new:prj'), exp='', file='common-two-diff')
+    @POST(rdiff_url('common-two', 'old:prj', 'new:prj'), exp='', file='common-two-diff')
     @POST(rdiff_url('common-three', 'old:prj', 'new:prj'), exp='', text='')
     def testPrdiffInclude(self):
         self._change_to_tmpdir()
@@ -200,9 +190,8 @@ identical: common-three
                                'old:prj', 'new:prj')
         self.assertEqualMultiline(out, exp)
 
-
     @GET_PROJECT_PACKAGES('old:prj', 'new:prj')
-    @POST(rdiff_url('common-two',   'old:prj', 'new:prj'), exp='', file='common-two-diff')
+    @POST(rdiff_url('common-two', 'old:prj', 'new:prj'), exp='', file='common-two-diff')
     @POST(rdiff_url('common-three', 'old:prj', 'new:prj'), exp='', text='')
     def testPrdiffExclude(self):
         self._change_to_tmpdir()
@@ -212,9 +201,8 @@ identical: common-three
         out = self._run_prdiff('--exclude', 'one', 'old:prj', 'new:prj')
         self.assertEqualMultiline(out, exp)
 
-
     @GET_PROJECT_PACKAGES('old:prj', 'new:prj')
-    @POST(rdiff_url('common-two',   'old:prj', 'new:prj'), exp='', file='common-two-diff')
+    @POST(rdiff_url('common-two', 'old:prj', 'new:prj'), exp='', file='common-two-diff')
     def testPrdiffIncludeExclude(self):
         self._change_to_tmpdir()
         exp = """differs:   common-two
@@ -224,19 +212,19 @@ identical: common-three
                                'old:prj', 'new:prj')
         self.assertEqualMultiline(out, exp)
 
-
     @GET_PROJECT_PACKAGES(UPSTREAM, BRANCH)
     @GET(request_url(UPSTREAM), exp='', file='request')
-    @POST(rdiff_url('common-one',   UPSTREAM, BRANCH), exp='', text='')
-    @POST(rdiff_url('common-two',   UPSTREAM, BRANCH), exp='', file='common-two-diff')
+    @POST(rdiff_url('common-one', UPSTREAM, BRANCH), exp='', text='')
+    @POST(rdiff_url('common-two', UPSTREAM, BRANCH), exp='', file='common-two-diff')
     @POST(rdiff_url('common-three', UPSTREAM, BRANCH), exp='', file='common-two-diff')
-    @POST(rdiff_url('only-in-new',  UPSTREAM, BRANCH), exp='', text='')
+    @POST(rdiff_url('only-in-new', UPSTREAM, BRANCH), exp='', text='')
     def testPrdiffRequestsMatching(self):
         self._change_to_tmpdir()
         exp = """identical: common-one
 differs:   common-two
 
 148023  State:new        By:user         When:2013-01-11T11:04:14
+        Created by: creator
         submit:          home:user:branches:some:project/common-two@7 ->    some:project
         Descr: - Fix it to work - Improve support for something
 
@@ -246,14 +234,14 @@ identical: only-in-new
         out = self._run_prdiff('--requests', UPSTREAM, BRANCH)
         self.assertEqualMultiline(out, exp)
 
-
     # Reverse the direction of the diff.
+
     @GET_PROJECT_PACKAGES(BRANCH, UPSTREAM)
     @GET(request_url(BRANCH), exp='', file='no-requests')
-    @POST(rdiff_url('common-one',   BRANCH, UPSTREAM), exp='', text='')
-    @POST(rdiff_url('common-two',   BRANCH, UPSTREAM), exp='', file='common-two-diff')
+    @POST(rdiff_url('common-one', BRANCH, UPSTREAM), exp='', text='')
+    @POST(rdiff_url('common-two', BRANCH, UPSTREAM), exp='', file='common-two-diff')
     @POST(rdiff_url('common-three', BRANCH, UPSTREAM), exp='', file='common-two-diff')
-    @POST(rdiff_url('only-in-new',  BRANCH, UPSTREAM), exp='', text='')
+    @POST(rdiff_url('only-in-new', BRANCH, UPSTREAM), exp='', text='')
     def testPrdiffRequestsSwitched(self):
         self._change_to_tmpdir()
         exp = """identical: common-one
@@ -265,7 +253,5 @@ identical: only-in-new
         self.assertEqualMultiline(out, exp)
 
 
-
 if __name__ == '__main__':
-    import unittest
     unittest.main()
